@@ -69,6 +69,11 @@ public:
         enableNoiseReduction      = 0;
         NRThresholdSpinBox        = 0;
         NRThresholdLabel          = 0;
+        enableCACorrection        = 0;
+        caRedMultSpinBox          = 0;
+        caBlueMultSpinBox         = 0;
+        caRedMultLabel            = 0;
+        caBlueMultLabel           = 0;
         unclipColorComboBox       = 0;
         reconstructLabel          = 0;
         reconstructSpinBox        = 0;
@@ -89,6 +94,8 @@ public:
     QLabel          *brightnessLabel;
     QLabel          *RAWQualityLabel;
     QLabel          *NRThresholdLabel;
+    QLabel          *caRedMultLabel;
+    QLabel          *caBlueMultLabel;
     QLabel          *unclipColorLabel;
     QLabel          *reconstructLabel;
     QLabel          *outputColorSpaceLabel;
@@ -109,11 +116,14 @@ public:
     QCheckBox       *autoColorBalanceCheckBox;
     QCheckBox       *dontStretchPixelsCheckBox;
     QCheckBox       *enableNoiseReduction;
+    QCheckBox       *enableCACorrection;
 
     KIntNumInput    *reconstructSpinBox;
     KIntNumInput    *blackPointSpinBox;
     KIntNumInput    *NRThresholdSpinBox;
- 
+
+    KDoubleNumInput *caRedMultSpinBox;
+    KDoubleNumInput *caBlueMultSpinBox;
     KDoubleNumInput *colorMult1SpinBox;
     KDoubleNumInput *colorMult2SpinBox;
     KDoubleNumInput *colorMult3SpinBox;
@@ -134,7 +144,7 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     // ---------------------------------------------------------------
 
     int line = 0;
-    
+
     d->sixteenBitsImage = new QCheckBox(i18n("16 bits color depth"), d->stdSettings);
     QWhatsThis::add( d->sixteenBitsImage, i18n("<p>If enabled, all RAW files will be decoded in 16-bit "
                                                "color depth using a linear gamma curve. To prevent dark "
@@ -144,7 +154,7 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
                                                "color depth with a BT.709 gamma curve and a 99th-percentile "
                                                "white point. This mode is faster than 16-bit decoding."));
     settingsBoxLayout->addMultiCellWidget(d->sixteenBitsImage, 0, 0, 0, 0);
-    
+
     if (sixteenBitsOption)
     {
         d->sixteenBitsImage->show();
@@ -194,8 +204,7 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
 
     d->autoColorBalanceCheckBox = new QCheckBox(i18n("Automatic color balance"), d->stdSettings);
     QWhatsThis::add( d->autoColorBalanceCheckBox, i18n("<p><b>Automatic color balance</b></p>"
-                                                  "The default is to use a fixed color balance "
-                                                  "based on a white card photographed in sunlight."));
+                                                  "Calculate the white balance by averaging the entire image."));
     settingsBoxLayout->addMultiCellWidget(d->autoColorBalanceCheckBox, line, line, 0, 2);    
     line++;
 
@@ -205,26 +214,28 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     d->unclipColorComboBox = new QComboBox( false, d->stdSettings );
     d->unclipColorComboBox->insertItem( i18n("Solid white"), 0 );
     d->unclipColorComboBox->insertItem( i18n("Unclip"),      1 );
-    d->unclipColorComboBox->insertItem( i18n("Reconstruct"), 2 );
+    d->unclipColorComboBox->insertItem( i18n("Blend"),       2 );
+    d->unclipColorComboBox->insertItem( i18n("Rebuild"),     3 );
     QWhatsThis::add( d->unclipColorComboBox, i18n("<p><b>Highlights</b><p>"
                                              "Select here the highlight clipping method:<p>"
                                              "<b>Solid white</b>: clip all highlights to solid white<p>"
                                              "<b>Unclip</b>: leave highlights unclipped in various "
                                              "shades of pink<p>"
-                                             "<b>Reconstruct</b>: reconstruct highlights using a "
-                                             "level value."));
-    settingsBoxLayout->addMultiCellWidget(d->unclipColorLabel, line, line, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->unclipColorComboBox, line, line, 1, 2);    
+                                             "<b>Blend</b>:Blend clipped and unclipped values together for a gradual fade to white<p>"
+                                             "<b>Rebuild</b>: reconstruct highlights using a "
+                                             "level value"));
+    settingsBoxLayout->addMultiCellWidget(d->unclipColorLabel, line, line, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->unclipColorComboBox, line, line, 1, 2);
     line++;
 
     d->reconstructLabel   = new QLabel(i18n("Level:"), d->stdSettings);
     d->reconstructSpinBox = new KIntNumInput(d->stdSettings);
-    d->reconstructSpinBox->setRange(0, 7, 1, true);
+    d->reconstructSpinBox->setRange(0, 6, 1, true);
     QWhatsThis::add(d->reconstructSpinBox, i18n("<p><b>Level</b><p>"
                                                "Specify the reconstruct highlight level. "
                                                "Low values favor whites and high values favor colors."));
-    settingsBoxLayout->addMultiCellWidget(d->reconstructLabel, line, line, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->reconstructSpinBox, line, line, 1, 2);    
+    settingsBoxLayout->addMultiCellWidget(d->reconstructLabel, line, line, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->reconstructSpinBox, line, line, 1, 2);
     line++;
 
     // ---------------------------------------------------------------
@@ -236,8 +247,8 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     QWhatsThis::add(d->brightnessSpinBox, i18n("<p><b>Brighness</b><p>"
                                                "Specify the brightness level of output image."
                                                "The default value is 1.0 (works in 8-bit mode only).<p>"));
-    settingsBoxLayout->addMultiCellWidget(d->brightnessLabel, line, line, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->brightnessSpinBox, line, line, 1, 2);    
+    settingsBoxLayout->addMultiCellWidget(d->brightnessLabel, line, line, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->brightnessSpinBox, line, line, 1, 2);
     line++;
 
     // ---------------------------------------------------------------
@@ -246,7 +257,8 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     d->RAWQualityComboBox = new QComboBox( false, d->stdSettings );
     d->RAWQualityComboBox->insertItem( i18n("Bilinear"), 0 );
     d->RAWQualityComboBox->insertItem( i18n("VNG"),      1 );
-    d->RAWQualityComboBox->insertItem( i18n("AHD"),      2 );
+    d->RAWQualityComboBox->insertItem( i18n("PPG"),      2 );
+    d->RAWQualityComboBox->insertItem( i18n("AHD"),      3 );
     QWhatsThis::add( d->RAWQualityComboBox, i18n("<p><b>Quality</b><p>"
                 "Select here the demosaicing RAW images decoding "
                 "interpolation method. A demosaicing algorithm is a digital image process used to "
@@ -262,6 +274,10 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
                 "This method computes gradients near the pixel of interest and uses "
                 "the lower gradients (representing smoother and more similar parts "
                 "of the image) to make an estimate.<p>"
+                "<b>PPG</b>: use Patterned Pixel Grouping interpolation. "
+                "Pixel Grouping uses assumptions about natural scenery in making estimates. "
+                "It has fewer color artifacts on natural images than the Variable Number of "
+                "Gradients method.<p>"
                 "<b>AHD</b>: use Adaptive Homogeneity-Directed interpolation. "
                 "This method selects the direction of interpolation so as to "
                 "maximize a homogeneity metric, thus typically minimizing color artifacts.<p>"));
@@ -288,6 +304,36 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
 
     // ---------------------------------------------------------------
 
+    d->enableCACorrection = new QCheckBox(i18n("Enable Chromatic Aberration correction"), d->stdSettings);
+    QWhatsThis::add( d->enableCACorrection, i18n("<p><b>Enable Chromatic Aberration correction</b><p>"
+                     "Enlarge the raw red and blue layers by the given factors, "
+                     "typically 0.999 to 1.001, to correct chromatic aberration.<p>"));
+    settingsBoxLayout->addMultiCellWidget(d->enableCACorrection, line, line, 0, 2);
+    line++;
+
+    d->caRedMultLabel   = new QLabel(i18n("Red multiplier:"), d->stdSettings);
+    d->caRedMultSpinBox = new KDoubleNumInput(d->stdSettings);
+    d->caRedMultSpinBox->setPrecision(5);
+    d->caRedMultSpinBox->setRange(0.00001, 2.0, 0.001, true);
+    QWhatsThis::add( d->caRedMultSpinBox, i18n("<p><b>Red multiplier</b><p>"
+                     "Set here the magnification factor of the red layer"));
+
+    d->caBlueMultLabel   = new QLabel(i18n("Blue multiplier:"), d->stdSettings);
+    d->caBlueMultSpinBox = new KDoubleNumInput(d->stdSettings);
+    d->caBlueMultSpinBox->setPrecision(5);
+    d->caBlueMultSpinBox->setRange(0.00001, 2.0, 0.001, true);
+    QWhatsThis::add( d->caBlueMultSpinBox, i18n("<p><b>Blue multiplier</b><p>"
+                     "Set here the magnification factor of the blue layer"));
+    
+    settingsBoxLayout->addMultiCellWidget(d->caRedMultLabel, line, line, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->caRedMultSpinBox, line, line, 1, 2);
+    line++;
+    settingsBoxLayout->addMultiCellWidget(d->caBlueMultLabel, line, line, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->caBlueMultSpinBox, line, line, 1, 2);
+    line++;
+
+    // ---------------------------------------------------------------
+
     d->outputColorSpaceLabel    = new QLabel(i18n("Color space:"), d->stdSettings);
     d->outputColorSpaceComboBox = new QComboBox( false, d->stdSettings );
     d->outputColorSpaceComboBox->insertItem( i18n("Raw (linear)"), 0 );
@@ -305,7 +351,7 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
                 "<b>Adobe RGB</b>: this color space is an extended RGB color space, developed by "
                 "Adobe. It is used for photography applications such as advertising "
                 "and fine art.<p>"
-                "<b>Wide Gamut</b>: this color space is is an expanded version of the "
+                "<b>Wide Gamut</b>: this color space is an expanded version of the "
                 "Adobe RGB color space.<p>"
                 "<b>Pro-Photo</b>: this color space is an RGB color space, developed by "
                 "Kodak, that offers an especially large gamut designed for use with "
@@ -317,12 +363,12 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     if (outputColorSpaceOption)
     {
         d->outputColorSpaceLabel->show();
-        d->outputColorSpaceComboBox->show(); 
+        d->outputColorSpaceComboBox->show();
     }
     else
     {
         d->outputColorSpaceLabel->hide();
-        d->outputColorSpaceComboBox->hide(); 
+        d->outputColorSpaceComboBox->hide();
     }
 
     insertTab(d->stdSettings, i18n("Standard"));
@@ -339,7 +385,7 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
                                                    "stretch the image to its correct aspect ratio. In any "
                                                    "case, this option guarantees that each output pixel "
                                                    "corresponds to one RAW pixel.<p>"));
-    settingsBoxLayout2->addMultiCellWidget(d->dontStretchPixelsCheckBox, 0, 0, 0, 2);   
+    settingsBoxLayout2->addMultiCellWidget(d->dontStretchPixelsCheckBox, 0, 0, 0, 2);
 
     // ---------------------------------------------------------------
 
@@ -352,8 +398,8 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     d->blackPointSpinBox->setRange(0, 1000, 1, true);
     QWhatsThis::add(d->blackPointSpinBox, i18n("<p><b>Black point value</b><p>"
                                                "Specify specific black point value of the output image.<p>"));
-    settingsBoxLayout2->addMultiCellWidget(d->blackPointCheckBox, 1, 1, 0, 0);    
-    settingsBoxLayout2->addMultiCellWidget(d->blackPointSpinBox, 1, 1, 1, 2);    
+    settingsBoxLayout2->addMultiCellWidget(d->blackPointCheckBox, 1, 1, 0, 0);
+    settingsBoxLayout2->addMultiCellWidget(d->blackPointSpinBox, 1, 1, 1, 2);
 
     // ---------------------------------------------------------------
 
@@ -379,19 +425,19 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     d->colorMult4SpinBox->setPrecision(5);
     d->colorMult4SpinBox->setRange(0.00001, 1.0, 0.01, true);
 
-    settingsBoxLayout->addMultiCellWidget(d->colorMultCheckBox, 2, 2, 0, 2);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult1Label, 3, 3, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult1SpinBox, 3, 3, 1, 2);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult2Label, 4, 4, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult2SpinBox, 4, 4, 1, 2);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult3Label, 5, 5, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult3SpinBox, 5, 5, 1, 2);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult4Label, 6, 6, 0, 0);    
-    settingsBoxLayout->addMultiCellWidget(d->colorMult4SpinBox, 6, 6, 1, 2);    
+    settingsBoxLayout->addMultiCellWidget(d->colorMultCheckBox, 2, 2, 0, 2);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult1Label, 3, 3, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult1SpinBox, 3, 3, 1, 2);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult2Label, 4, 4, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult2SpinBox, 4, 4, 1, 2);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult3Label, 5, 5, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult3SpinBox, 5, 5, 1, 2);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult4Label, 6, 6, 0, 0);
+    settingsBoxLayout->addMultiCellWidget(d->colorMult4SpinBox, 6, 6, 1, 2);
 
-    settingsBoxLayout2->setRowStretch(7, 10);   
+    settingsBoxLayout2->setRowStretch(7, 10);
     insertTab(d->advSettings, i18n("Advanced"));
-    
+
     if (!showAdvancedOptions)
     {
         removePage(d->advSettings);
@@ -406,6 +452,9 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
     connect(d->enableNoiseReduction, SIGNAL(toggled(bool)),
             this, SLOT(slotNoiseReductionToggled(bool)));
 
+    connect(d->enableCACorrection, SIGNAL(toggled(bool)),
+            this, SLOT(slotCACorrectionToggled(bool)));
+
     connect(d->blackPointCheckBox, SIGNAL(toggled(bool)),
             d->blackPointSpinBox, SLOT(setEnabled(bool)));
 
@@ -437,6 +486,9 @@ void DcrawSettingsWidget::setDefaultSettings()
     setUnclipColor(0);
     setDontStretchPixels(false);
     setNoiseReduction(false);
+    setUseCACorrection(false);
+    setcaRedMultiplier(1.0);
+    setcaBlueMultiplier(1.0);
     setBrightness(1.0);
     setUseBlackPoint(false);
     setBlackPoint(0);
@@ -446,8 +498,8 @@ void DcrawSettingsWidget::setDefaultSettings()
     setcolorMultiplier3(1.0);
     setcolorMultiplier4(1.0);
     setNRThreshold(100);
-    setQuality(RawDecodingSettings::BILINEAR); 
-    setOutputColorSpace(RawDecodingSettings::SRGB); 
+    setQuality(RawDecodingSettings::BILINEAR);
+    setOutputColorSpace(RawDecodingSettings::SRGB);
 }
 
 void DcrawSettingsWidget::slotsixteenBitsImageToggled(bool b)
@@ -459,7 +511,7 @@ void DcrawSettingsWidget::slotsixteenBitsImageToggled(bool b)
 
 void DcrawSettingsWidget::slotUnclipColorActivated(int v)
 {
-    if (v == 2)     // Reconstruct Highlight method
+    if (v == 3)     // Reconstruct Highlight method
     {
         d->reconstructLabel->setEnabled(true);
         d->reconstructSpinBox->setEnabled(true);
@@ -477,6 +529,14 @@ void DcrawSettingsWidget::slotNoiseReductionToggled(bool b)
     d->NRThresholdLabel->setEnabled(b);
 }
 
+void DcrawSettingsWidget::slotCACorrectionToggled(bool b)
+{
+    d->caRedMultSpinBox->setEnabled(b);
+    d->caBlueMultSpinBox->setEnabled(b);
+    d->caRedMultLabel->setEnabled(b);
+    d->caBlueMultLabel->setEnabled(b);
+}
+
 void DcrawSettingsWidget::slotColorMultToggled(bool b)
 {
     d->colorMult1SpinBox->setEnabled(b);
@@ -549,8 +609,11 @@ int DcrawSettingsWidget::unclipColor()
         case 1:
             return 1;
             break;
+        case 2:
+            return 2;
+            break;
         default:         // Reconstruct Highlight method
-            return d->reconstructSpinBox->value()+2;
+            return d->reconstructSpinBox->value()+3;
             break;
     }
 }
@@ -565,9 +628,12 @@ void DcrawSettingsWidget::setUnclipColor(int v)
         case 1:
             d->unclipColorComboBox->setCurrentItem(1);
             break;
-        default:         // Reconstruct Highlight method
+        case 2:
             d->unclipColorComboBox->setCurrentItem(2);
-            d->reconstructSpinBox->setValue(v-2);
+            break;
+        default:         // Reconstruct Highlight method
+            d->unclipColorComboBox->setCurrentItem(3);
+            d->reconstructSpinBox->setValue(v-3);
             break;
     }
 
@@ -633,9 +699,12 @@ RawDecodingSettings::DecodingQuality DcrawSettingsWidget::quality()
             return RawDecodingSettings::VNG;
             break;
         case 2:
+            return RawDecodingSettings::PPG;
+            break;
+        case 3:
             return RawDecodingSettings::AHD;
             break;
-        default:    
+        default:
             return RawDecodingSettings::BILINEAR;
             break;
     }
@@ -648,10 +717,13 @@ void DcrawSettingsWidget::setQuality(RawDecodingSettings::DecodingQuality q)
         case RawDecodingSettings::VNG:
             d->RAWQualityComboBox->setCurrentItem(1);
             break;
-        case RawDecodingSettings::AHD:
+        case RawDecodingSettings::PPG:
             d->RAWQualityComboBox->setCurrentItem(2);
             break;
-        default:    
+        case RawDecodingSettings::AHD:
+            d->RAWQualityComboBox->setCurrentItem(3);
+            break;
+        default:
             d->RAWQualityComboBox->setCurrentItem(0);
             break;
     }
@@ -696,6 +768,43 @@ void DcrawSettingsWidget::setNRThreshold(int b)
 
 // ---------------------------------------------------------------
 
+bool DcrawSettingsWidget::useCACorrection()
+{
+    return d->enableCACorrection->isChecked();
+}
+
+void DcrawSettingsWidget::setUseCACorrection(bool b)
+{
+    d->enableCACorrection->setChecked(b);
+    slotCACorrectionToggled(b);
+}
+
+// ---------------------------------------------------------------
+
+double DcrawSettingsWidget::caRedMultiplier()
+{
+    return d->caRedMultSpinBox->value();
+}
+
+void DcrawSettingsWidget::setcaRedMultiplier(double b)
+{
+    d->caRedMultSpinBox->setValue(b);
+}
+
+// ---------------------------------------------------------------
+
+double DcrawSettingsWidget::caBlueMultiplier()
+{
+    return d->caBlueMultSpinBox->value();
+}
+
+void DcrawSettingsWidget::setcaBlueMultiplier(double b)
+{
+    d->caBlueMultSpinBox->setValue(b);
+}
+
+// ---------------------------------------------------------------
+
 bool DcrawSettingsWidget::useColorMultipliers()
 {
     return d->colorMultCheckBox->isChecked();
